#include <vector>
#include "ElementMatrix.h"
#include "ElementVector.h"
#include "SubElementAssembler.h"
#include "ScalableQuadrature.h"
#include "SubPolytope.h"

SubElementAssembler::SubElementAssembler(Operator *op, 
		      const FiniteElemSpace *rowFESpace_,
		      const FiniteElemSpace *colFESpace_)
  : StandardAssembler(op, NULL, NULL, NULL, NULL, rowFESpace_, colFESpace_)
{
  /** 
   * Create a scalable quadrature for subassembler and replace the original 
   * quadrature of the subassembler with the scalable quadrature.
   */

  if (zeroOrderAssembler) {
    checkQuadratures();
    zeroOrderScalableQuadrature = 
                    NEW ScalableQuadrature(zeroOrderAssembler->getQuadrature());
    zeroOrderAssembler->setQuadrature(zeroOrderScalableQuadrature);
  }
  else {
    zeroOrderScalableQuadrature = NULL;
  }

  if (firstOrderAssemblerGrdPsi) {
    checkQuadratures();
    firstOrderGrdPsiScalableQuadrature = 
                    NEW ScalableQuadrature(firstOrderAssemblerGrdPsi->getQuadrature());
    firstOrderAssemblerGrdPsi->setQuadrature(firstOrderGrdPsiScalableQuadrature);
  }
  else {
    firstOrderGrdPsiScalableQuadrature = NULL;
  }

  if (firstOrderAssemblerGrdPhi) {
    checkQuadratures();
    firstOrderGrdPhiScalableQuadrature = 
                    NEW ScalableQuadrature(firstOrderAssemblerGrdPhi->getQuadrature());
    firstOrderAssemblerGrdPhi->setQuadrature(firstOrderGrdPhiScalableQuadrature);
  }
  else {
    firstOrderGrdPhiScalableQuadrature = NULL;
  }

  if (secondOrderAssembler) {
    checkQuadratures();
    secondOrderScalableQuadrature = 
                    NEW ScalableQuadrature(secondOrderAssembler->getQuadrature());
    secondOrderAssembler->setQuadrature(secondOrderScalableQuadrature);
  }
  else {
    secondOrderScalableQuadrature = NULL;
  }
}

void
SubElementAssembler::scaleQuadratures(const SubElInfo& subElInfo)
{
  if (zeroOrderAssembler) {
    zeroOrderScalableQuadrature->scaleQuadrature(subElInfo);
  }
  if (firstOrderAssemblerGrdPsi) {
    firstOrderGrdPsiScalableQuadrature->scaleQuadrature(subElInfo);
  }
  if (firstOrderAssemblerGrdPhi) {
    firstOrderGrdPhiScalableQuadrature->scaleQuadrature(subElInfo);
  }
  if (secondOrderAssembler) {
    secondOrderScalableQuadrature->scaleQuadrature(subElInfo);
  }
}

void 
SubElementAssembler::getSubElementVector(SubElInfo *subElInfo, 
                                         const ElInfo *elInfo, 
                                         ElementVector *userVec)
{
  double corrFactor;

  /** 
   * Manipulate the quadratures of the SubAssemblers for subelement.
   */
  scaleQuadratures(*subElInfo);

  calculateElementVector(elInfo, userVec);

  /** 
   * The integration has been performed with a quadrature living on element. The 
   * determinant of element has been used instead of the determinant of subelement. Thus
   * the result must be corrected with respect to subelement.
   */
  corrFactor = subElInfo->getDet() / fabs(elInfo->getDet());
  int i;
  for ( i = 0; i < nRow; i++) {
    (*userVec)[i] *= corrFactor;
  }
}

void 
SubElementAssembler::getSubElementMatrix(SubElInfo *subElInfo, 
                                         const ElInfo *elInfo, 
                                         ElementMatrix *userMat)
{
  double corrFactor;

  /** 
   * Manipulate the quadratures of the SubAssemblers for subelement.
   */
  scaleQuadratures(*subElInfo);

  /** 
   * Integrate using the manipulated quadrature.
   */
  calculateElementMatrix(elInfo, userMat);

  /** 
   * The integration has been performed with a quadrature living on element. The 
   * determinant of element has been used instead of the determinant of subelement. 
   * Thus the result must be corrected with respect to subelement.
   */
  corrFactor = subElInfo->getDet() / fabs(elInfo->getDet());
  int i, j;
  for ( i = 0; i < nRow; i++) {
    for ( j = 0; j < nCol; j++) {
      (*userMat)[i][j] *= corrFactor;
    }
  }
}

void
SubElementAssembler::getSubPolytopeVector(SubPolytope *subPolytope,
					  SubElementAssembler *subElementAssembler,
					  const ElInfo *elInfo,
					  ElementVector *subPolVec)
{
  /**
   * Note: There is no reset of subPolVec.
   */
  std::vector<SubElInfo *>::iterator it;
  ElementVector *subElVec = NEW ElementVector(nRow);
  int i;

  /**
   * Assemble for each subelement of subpolytope.
   */
  for (it = subPolytope->getSubElementsBegin(); 
       it != subPolytope->getSubElementsEnd();
       it++) {
    subElVec->set(0.0);
    subElementAssembler->getSubElementVector(*it, elInfo, subElVec);

    /**
     * Add results for subelement to total result for subpolytope.
     */
    for(i = 0; i < nRow; i++) {
      (*subPolVec)[i] += (*subElVec)[i];
    }
  }

  DELETE subElVec;
}

void
SubElementAssembler::getSubPolytopeMatrix(SubPolytope *subPolytope,
					  SubElementAssembler *subElementAssembler,
					  const ElInfo *elInfo,
					  ElementMatrix *subPolMat)
{
  /**
   * Note: There is no reset of subPolMat.
   */
  std::vector<SubElInfo *>::iterator it;
  ElementMatrix *subElMat = NEW ElementMatrix(nRow, nCol);
  int i,j;

  /**
   * Assemble for each subelement of subpolytope.
   */
  for (it = subPolytope->getSubElementsBegin(); 
       it != subPolytope->getSubElementsEnd();
       it++) {
    subElMat->set(0.0);
    subElementAssembler->getSubElementMatrix(*it, elInfo, subElMat);

    /**
     * Add results for subelement to total result for subpolytope.
     */
    for(i = 0; i < nRow; i++) {
      for(j = 0; j < nCol; j++) {
	(*subPolMat)[i][j] += (*subElMat)[i][j];
      }
    }
  }

  DELETE subElMat;
}
