// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  crystal growth group                                                  ==
// ==                                                                        ==
// ==  Stiftung caesar                                                       ==
// ==  Ludwig-Erhard-Allee 2                                                 ==
// ==  53175 Bonn                                                            ==
// ==  germany                                                               ==
// ==                                                                        ==
// ============================================================================
// ==                                                                        ==
// ==  http://www.caesar.de/cg/AMDiS                                         ==
// ==                                                                        ==
// ============================================================================

/** \file ProblemIterationInterface.h */

#ifndef AMDIS_PROBLEMITERATIONINTERFACE_H
#define AMDIS_PROBLEMITERATIONINTERFACE_H

#include "Flag.h"

namespace AMDiS {

  class AdaptInfo;
  class ProblemStatBase;

  const Flag BUILD =    0x01L;
  const Flag ADAPT =    0x02L;
  const Flag SOLVE =    0x04L;
  const Flag ESTIMATE = 0x08L;
  const Flag MARK =     0x10L;

  const Flag FULL_ITERATION = BUILD | ADAPT | SOLVE | ESTIMATE | MARK;
  const Flag NO_ADAPTION = BUILD | SOLVE | ESTIMATE;

  // ============================================================================
  // ===== class ProblemIterationInterface ======================================
  // ============================================================================

  /** \brief
   * Interface for master problems needed by the adaption loop. A master problem
   * can handle one single or multiple coupled problems. In the latter case,
   * the master problem can determine the execution order of the build, solve,
   * estimate, and adapt steps of the single problems in \ref oneIteration().
   */
  class ProblemIterationInterface
  {
  public:
    virtual ~ProblemIterationInterface() {};

    /** \brief
     * Called before each adaption loop iteration.
     */
    virtual void beginIteration(AdaptInfo *adaptInfo) {};

    /** \brief
     * Determines the execution order of the single adaption steps. If adapt is
     * true, mesh adaption will be performed. This allows to avoid mesh adaption,
     * e.g. in timestep adaption loops of timestep adaptive strategies.
     */
    virtual Flag oneIteration(AdaptInfo *adaptInfo, Flag toDo = FULL_ITERATION) = 0;

    /** \brief
     * Called after each adaption loop iteration.
     */
    virtual void endIteration(AdaptInfo *adaptInfo) {};

    /** \brief
     * Returns number of managed problems
     */
    virtual int getNumProblems() = 0;

    /** \brief
     * Returns the problem with the given number. If only one problem
     * is managed by this master problem, the number hasn't to be given.
     */
    virtual ProblemStatBase *getProblem(int number = 0) = 0;

    /** \brief
     * Returns the problem with the given name. 
     */
    virtual ProblemStatBase *getProblem(const ::std::string& name) { return NULL; };

    /** \brief
     * Returns the name of the problem.
     */
    virtual const ::std::string& getName() = 0;

    /** \brief
     * Function that serializes the problem plus information about the iteration.
     */
    virtual void serialize(::std::ostream &out) = 0;

    /** \brief
     * Function that deserializes the problem plus information about the iteration.
     */
    virtual void deserialize(::std::istream &in) = 0;
  };

}

#endif

